Option Explicit On
Imports SySimFWK, PredefinedControls

Namespace EULYNX_Profile.Data_Types
  ' Entity 'Message' not generated by SySim because unreferenced or not having qualifying stereotypes
  ' Entity 'Permanent_Information' not generated by SySim because unreferenced or not having qualifying stereotypes
  <Serializable()> Public Class PulsedIn : Implements ICopyable, ITextable 
    Private PulseCount As Integer
    Private LastCopyableBooleanOrigin As Boolean
    Private Masked As Boolean
    Public Function CopyObject(ByVal POrigin As Object) As Boolean
      ' ## VBOperationBody [06e6c4ef-942c-4016-a71a-794e9f538f9c] 
      Dim Ret As Boolean = False
      If TypeOf POrigin Is PulsedOut Then
        Dim OldPulseCount As Integer = PulseCount
        PulseCount += CType(POrigin, PulsedOut).GetPulseCount()
        If PulseCount <> OldPulseCount Then
          Ret = True
        End If
      Else ' must be boolean
        Dim Origin As Boolean = CType(POrigin, Boolean)
        If Origin And Not LastCopyableBooleanOrigin Then
          PulseCount += 1
          Ret = True
        End If
        LastCopyableBooleanOrigin = Origin
      End If
      Return Ret
      ' ## VBOperationBody End 
    End Function
    
    Public Function Decrement() As Boolean
      ' ## VBOperationBody [b7e3d628-fd95-40ca-9b45-686d0278ce04] 
      Dim Ret As Boolean = False
      If PulseCount > 0 Then
        PulseCount -= 1
        If PulseCount > 0 Then
          Ret = True
        End If
      End If
      Return Ret
      ' ## VBOperationBody End 
    End Function
    
    Public Sub Reset()
      ' ## VBOperationBody [c4046c72-ea13-4032-95fb-c743ac5eca5f] 
      PulseCount = 0
      ' ## VBOperationBody End 
    End Sub
    
    Public Sub Mask(ByVal What As Boolean)
      ' ## VBOperationBody [75c013fe-5036-4cc8-99a3-778007fa962c] 
      Masked = What
      ' ## VBOperationBody End 
    End Sub
    
    Public Shared Widening Operator CType(ByVal A As PulsedIn) As Boolean
      ' ## VBOperationBody [9a98197e-12d0-4968-8bb8-21a73d56ba39] 
      Dim Ret As Boolean = False
      If Not A.Masked Then
        Ret = (A.PulseCount > 0)
      End If
      Return Ret
      ' ## VBOperationBody End 
    End Operator
    
    Public Shared Operator Or(ByVal A As PulsedIn, ByVal B As PulsedIn) As Boolean
      ' ## VBOperationBody [b91aa478-2ee2-4443-9781-d411031adac7] 
      Return CType(A, Boolean) Or CType(B, Boolean)
      ' ## VBOperationBody End 
    End Operator
    
    Public Shared Operator Or(ByVal A As Boolean, ByVal B As PulsedIn) As Boolean
      ' ## VBOperationBody [a8ffd64d-0d16-4960-90cd-2d7df95dc3ca] 
      Return A Or CType(B, Boolean)
      ' ## VBOperationBody End 
    End Operator
    
    Public Shared Operator Or(ByVal A As PulsedIn, ByVal B As Boolean) As Boolean
      ' ## VBOperationBody [2a879a3e-3d8a-458f-bce5-946c30ec956d] 
      Return CType(A, Boolean) Or B
      ' ## VBOperationBody End 
    End Operator
    
    Public Shared Operator And(ByVal A As PulsedIn, ByVal B As PulsedIn) As Boolean
      ' ## VBOperationBody [773599ec-b0a2-42ff-958a-0c0af7d8da28] 
      Return CType(A, Boolean) And CType(B, Boolean)
      ' ## VBOperationBody End 
    End Operator
    
    Public Shared Operator Not(ByVal O As PulsedIn) As Boolean
      ' ## VBOperationBody [25cbf1e9-1c28-434b-afad-c465cd4cf4e0] 
      Return Not CType(O, Boolean)
      ' ## VBOperationBody End 
    End Operator
    
    Public Shared Overloads Operator <>(ByVal B As EULYNX_Profile.Data_Types.PulsedIn, ByVal A As EULYNX_Profile.Data_Types.PulsedIn) As Boolean
      Return Not (A = B)
    End Operator
    
    Public Overridable Function Copy(ByVal POrigin As ICopyable) As Boolean Implements ICopyable.Copy
      Dim Ret As Boolean = False
      If POrigin IsNot Nothing Then
        Dim Origin As PulsedIn = POrigin
        If PulseCount <> Origin.PulseCount Then
          PulseCount = Origin.PulseCount
          Ret = True
        End If
        If LastCopyableBooleanOrigin <> Origin.LastCopyableBooleanOrigin Then
          LastCopyableBooleanOrigin = Origin.LastCopyableBooleanOrigin
          Ret = True
        End If
        If Masked <> Origin.Masked Then
          Masked = Origin.Masked
          Ret = True
        End If
      End If
      Return Ret
    End Function
    
    Public Overridable Sub ToText(ByRef Txt As String, ByVal ShowContainment As Boolean) Implements ITextable.ToText
      If  ShowContainment Then Txt += "("
      If Txt <> "" AndAlso Not Txt.EndsWith("(") Then Txt += ","
      ODataUtils.ToText(PulseCount, Txt, False)
      If Txt <> "" AndAlso Not Txt.EndsWith("(") Then Txt += ","
      ODataUtils.ToText(LastCopyableBooleanOrigin, Txt, False)
      If Txt <> "" AndAlso Not Txt.EndsWith("(") Then Txt += ","
      ODataUtils.ToText(Masked, Txt, False)
      If ShowContainment Then Txt +=")"
      
    End Sub
    
    Public Shared Overloads Operator =(ByVal B As EULYNX_Profile.Data_Types.PulsedIn, ByVal A As EULYNX_Profile.Data_Types.PulsedIn) As Boolean
      If A.PulseCount <> B.PulseCount Then Return False
      If A.LastCopyableBooleanOrigin <> B.LastCopyableBooleanOrigin Then Return False
      If A.Masked <> B.Masked Then Return False
      Return True
    End Operator
    
  End Class
  <Serializable()> Public Class PulsedOut : Implements ICopyable, ITextable 
    Private PulseCount As Integer
    Public Function CopyObject(ByVal POrigin As Object) As Boolean
      ' ## VBOperationBody [a8839050-f649-45e0-b1df-726389925322] 
      Dim Ret As Boolean = False
      Dim Origin As Boolean
      If TypeOf POrigin Is PulsedOut Then
        ' Add the count
        Dim OldPulseCount As Integer = PulseCount
        PulseCount += CType(POrigin, PulsedOut).PulseCount
        If PulseCount <> OldPulseCount Then
          Ret = True
        End If
      Else ' Must be boolean
        Origin = CType(POrigin, Boolean)
        If Origin Then
          PulseCount += 1
          Ret = True
        End If
      End If
      Return Ret
      ' ## VBOperationBody End 
    End Function
    
    Public Shared Widening Operator CType(ByVal A As Boolean) As PulsedOut
      ' ## VBOperationBody [687ab83c-0111-41a4-a1ea-2238ffb05b7c] 
      Dim Ret As New PulsedOut
      If A Then
        Ret.PulseCount = 1
      End If
      Return Ret
      ' ## VBOperationBody End 
    End Operator
    
    Public Sub Reset()
      ' ## VBOperationBody [cac9b444-6912-4a82-a66f-e78d4ecc374c] 
      PulseCount = 0
      ' ## VBOperationBody End 
    End Sub
    
    Public Shared Overloads Operator <>(ByVal B As EULYNX_Profile.Data_Types.PulsedOut, ByVal A As EULYNX_Profile.Data_Types.PulsedOut) As Boolean
      Return Not (A = B)
    End Operator
    
    Public Overridable Function Copy(ByVal POrigin As ICopyable) As Boolean Implements ICopyable.Copy
      Dim Ret As Boolean = False
      If POrigin IsNot Nothing Then
        Dim Origin As PulsedOut = POrigin
        If PulseCount <> Origin.PulseCount Then
          PulseCount = Origin.PulseCount
          Ret = True
        End If
      End If
      Return Ret
    End Function
    
    Public Overridable Sub ToText(ByRef Txt As String, ByVal ShowContainment As Boolean) Implements ITextable.ToText
      If  ShowContainment Then Txt += "("
      If Txt <> "" AndAlso Not Txt.EndsWith("(") Then Txt += ","
      ODataUtils.ToText(PulseCount, Txt, False)
      If ShowContainment Then Txt +=")"
      
    End Sub
    
    Public Shared Overloads Operator =(ByVal B As EULYNX_Profile.Data_Types.PulsedOut, ByVal A As EULYNX_Profile.Data_Types.PulsedOut) As Boolean
      If A.PulseCount <> B.PulseCount Then Return False
      Return True
    End Operator
    
    Public Function GetPulseCount() As Integer
      Return PulseCount
    End Function
    
  End Class
End Namespace
