﻿Imports Testing
Module Main
	Sub Main(args As String())
		Dim filename As String = Nothing
		Dim stabilizationCount As Integer = 20
		Dim valuePerTag As Dictionary(Of String, String) = New Dictionary(Of String, String)
		For Each arg In args
			If arg.StartsWith("-") Then
				Dim xs As String() = arg.Split("=")
				If xs.Length > 1 Then
					valuePerTag.Add(xs.GetValue(0), xs.GetValue(1))
				Else
					valuePerTag.Add(arg, "1")
				End If
			ElseIf filename Is Nothing Then
				filename = arg
			Else
				Throw New Exception("Found more than one filename")
			End If
		Next
		If filename Is Nothing Then
			Console.WriteLine("(no filename provided by user, using default)")
			filename = "point.bisim.t77.sic"
			'filename = "point.ie-off.bisim.tsuite5.sic"
		End If
		Dim testSuiteName As String = "trcov"
		If valuePerTag.TryGetValue("--testsuite", testSuiteName) OrElse valuePerTag.TryGetValue("-T", testSuiteName) Then
			'Do nothing
		Else
			Console.WriteLine("(no test suite provided by user, using default)")
		End If
		Dim applyHiddenTimeouts As Boolean = False
		Dim tagValue As String = Nothing
		If valuePerTag.TryGetValue("--help", tagValue) OrElse valuePerTag.TryGetValue("-h", tagValue) Then
			If Not tagValue.Equals("0") Then
				Console.WriteLine("Arguments: [tags] [filename]")
				Console.WriteLine("Tags:")
				Console.WriteLine("  --help               -h    Print usage")
				Console.WriteLine("  --stab=<int>         -s    Number of stabilization steps after each input (default: 20)")
				Console.WriteLine("  --testsuite=<name>   -T    Name of the test suite that should be applied (default: trcov)")
				Console.WriteLine("  --timeouts           -u    Apply unspecified timeouts (default: False)")
				Return 'Stop here
			End If
		End If
		If valuePerTag.TryGetValue("--stab", tagValue) OrElse valuePerTag.TryGetValue("-s", tagValue) Then
			Dim i As Integer
			If Int32.TryParse(tagValue, i) AndAlso i >= 1 Then
				stabilizationCount = i
			Else
				Throw New Exception("Invalid count (" + tagValue + ")")
			End If
		End If
		If valuePerTag.TryGetValue("--timeouts", tagValue) OrElse valuePerTag.TryGetValue("-u", tagValue) Then
			applyHiddenTimeouts = True
		End If
		Console.WriteLine("Version: 2025-01-27")
		Console.WriteLine("File: " + filename)
		Console.WriteLine("Test suite: " + testSuiteName)
		Console.WriteLine("Stabilization count: " + stabilizationCount.ToString)
		Console.WriteLine("Hidden timeout application: " + applyHiddenTimeouts.ToString)
		Dim model = New LoadedModel()
		model.LoadFromFile(filename)
		Dim testAPI As TestAPI = New TestAPI(stabilizationCount, applyHiddenTimeouts)
		Dim testSuite As LoadedModelTestSuite = New LoadedModelTestSuite(model, testSuiteName)
		testSuite.Run(testAPI)
	End Sub
End Module
