﻿Namespace Testing
	Public Class ConformanceException
		Inherits System.ApplicationException
		Public Sub New(ByVal message As String)
			MyBase.New(message)
		End Sub
	End Class
	Public Interface Test
		Function GetName() As String
		Function GetTotalStepCount() As Integer
		Function GetDoneStepCount() As Integer
		Sub Run(testAPI As TestAPI)
	End Interface
	Public Class TestDiscrepancy
		Public ReadOnly header As String
		Public ReadOnly expected As String
		Public ReadOnly observed As String
		Public Sub New(header As String, expected As String, found As String)
			Me.header = header
			Me.expected = expected
			Me.observed = found
		End Sub
		Public Overrides Function Equals(obj As Object) As Boolean
			Dim d = TryCast(obj, TestDiscrepancy)
			Return d IsNot Nothing AndAlso header = d.header AndAlso expected = d.expected AndAlso observed = d.observed
		End Function
		Public Overrides Function GetHashCode() As Integer
			Dim hashCode As Long = -442547264
			hashCode = (hashCode * -1521134295 + EqualityComparer(Of String).Default.GetHashCode(header)).GetHashCode()
			hashCode = (hashCode * -1521134295 + EqualityComparer(Of String).Default.GetHashCode(expected)).GetHashCode()
			hashCode = (hashCode * -1521134295 + EqualityComparer(Of String).Default.GetHashCode(observed)).GetHashCode()
			Return hashCode
		End Function
	End Class
	Public Class TestSuiteResult
		Public passCount As Integer
		Public failCount As Integer
		Public testCount As Integer
	End Class
	Public MustInherit Class TestSuite
		Private discrepancies As Dictionary(Of TestDiscrepancy, HashSet(Of String))
		Public Sub New()
			discrepancies = New Dictionary(Of TestDiscrepancy, HashSet(Of String))
		End Sub
		Public MustOverride Function GetTests() As Test()
		Public Function Run(testAPI As TestAPI) As TestSuiteResult
			Dim result As TestSuiteResult = New TestSuiteResult()
			Dim tests As Test() = GetTests()
			Dim startTime As DateTime = Now
			Dim processedStepCount As Integer = 0
			Dim skippedStepCount As Integer = 0
			Dim totalStepCount As Integer = 0
			For Each test As Test In tests
				totalStepCount = totalStepCount + test.GetTotalStepCount()
			Next
			For Each test As Test In tests
				Try
					testAPI.SetStepCounts(processedStepCount, skippedStepCount, totalStepCount)
					testAPI.SetPrefix(String.Format(" [{0}({1}) failures][{2}/{3}]", result.failCount, discrepancies.Count, result.passCount + result.failCount + 1, tests.Length))
					test.Run(testAPI)
					result.passCount = result.passCount + 1
				Catch e As ConformanceException
					result.failCount = result.failCount + 1
					skippedStepCount = skippedStepCount + test.GetTotalStepCount() - test.GetDoneStepCount()
					Dim saveThisTest As Boolean = False
					Dim saveSummary As Boolean = False
					For Each d In testAPI.GetDiscrepancies()
						Dim ts As HashSet(Of String) = Nothing
						If Not discrepancies.TryGetValue(d, ts) Then
							ts = New HashSet(Of String)
							discrepancies.Add(d, ts)
						End If
						If ts.Count < 3 Then 'Limit number of failure examples
							ts.Add(test.GetName() + ".fail")
							saveThisTest = True
							saveSummary = True
						Else
							If ts.Count < 4 Then
								ts.Add(". . .") 'Make clear that there are more instances
								saveSummary = True
							End If
						End If
					Next
					If saveThisTest Then
						testAPI.SaveLog(test.GetName() + ".fail")
					End If
					If saveSummary Then
						SaveDiscrepancies("discrepancies.txt")
					End If
				End Try
				result.testCount = result.testCount + 1
				processedStepCount = processedStepCount + test.GetTotalStepCount()
			Next
			Dim endTime As DateTime = Now
			testAPI.Log("Final summary:")
			testAPI.Log(String.Format(" Applied {0} test steps in " + endTime.Subtract(startTime).TotalSeconds.ToString("0.000") + "s", totalStepCount - skippedStepCount))
			testAPI.Log(String.Format(" Could not reach {0} test steps because of failures", skippedStepCount))
			Return result
		End Function
		Public Sub SaveDiscrepancies(filename As String)
			Dim out As New System.IO.StreamWriter(filename, False)
			For Each d In discrepancies
				out.WriteLine(d.Key.header + ":")
				out.WriteLine("  Expected: " + d.Key.expected)
				out.WriteLine("  Observed: " + d.Key.observed)
				out.WriteLine("Tests:")
				For Each t In d.Value
					out.WriteLine("  " + t)
				Next
				out.WriteLine()
			Next
			out.Close()
		End Sub
	End Class
End Namespace
